@@ -22,7 +22,7 @@ package Net::FullAuto::ISets::Amazon::Chef_is;
 #######################################################################
 
 our $VERSION='0.01';
-our $DISPLAY='Chef High Availability';
+our $DISPLAY='Chef® High Availability';
 our $CONNECT='ssh';
 our $defaultInstanceType='t2.small';
 
@@ -47,9 +47,8 @@ my $configure_mysql=sub {
    my $lr_inst=$main::aws->{'Chef.io'}->[$cnt]->[0];
    my $handle=$main::aws->{$server_type}->[$cnt]->[1];
    my ($stdout,$stderr)=('','');
-   ($stdout,$stderr)=$handle->cmd("sudo yum clean all");
-   ($stdout,$stderr)=$handle->cmd("sudo yum grouplist hidden");
-   ($stdout,$stderr)=$handle->cmd("sudo yum groups mark convert");
+   ($stdout,$stderr)=$handle->cmd(
+      "sudo apt-get -y -o Dpkg::Progress=true update 2>&1",'__display__');
    my $install_mysql=<<'END';
 
           o o    o .oPYo. ooooo    .oo o     o     o o    o .oPYo.
@@ -284,23 +283,50 @@ my $configure_chef=sub {
    my $server_host_block=$_[3]||'';
    my $handle=$main::aws->{$server_type}->[$cnt]->[1];
    my ($stdout,$stderr,$url)=(''.''.'');
-   ($stdout,$stderr)=$handle->cmd('sudo yum -y install epel-release',
-      '__display__');
+   ($stdout,$stderr)=$handle->cmd(
+      "sudo apt-get -y -o Dpkg::Progress=true update 2>&1",'__display__');
+   ($stdout,$stderr)=$handle->cmd('sudo '.
+      'apt-get -y -o Dpkg::Progress=true install gnupg2','__display__');
    ($stdout,$stderr)=$handle->cmd(
       'gpg2 --keyserver hkp://keys.gnupg.net --recv-keys '.
       '409B6B1796C275462A1703113804BB82D39DC0E3','__display__');
    ($stdout,$stderr)=$handle->cmd('sudo '.
-      'yum -y install autoconf automake gcc gcc-c++ git libtool make nasm pkgconfig wget '.
-      'patch readline readline-devel zlib zlib-devel libyaml-devel libffi-devel openssl-devel'.
-      'bzip2 libtool bison iconv-devel ruby-devel libxml2 libxml2-devel libxslt libxslt-devel',
-      '__display__');
+      'sudo apt-get -y -o Dpkg::Progress=true install autoconf automake gcc '.
+      'g++ libtool libyaml-dev make nasm pkg-config wget flex patch '.
+      'libreadline6 libreadline6-dev zlibc zlib1g zlib1g-dev libsqlite3-dev '.
+      'libffi-dev libssl-dev bzip2 libtool bison ruby-dev libxml2 git zip '.
+      'rebar rabbitmq-server python-dev xsltproc libxml2-dev libxslt1-dev '.
+      'build-essential libssh-dev unixodbc-dev libpq-dev',
+      '__display__'); # erlang erlang-eunit
    ($stdout,$stderr)=$handle->cmd(
-      'curl -sSL https://get.rvm.io | bash -s stable --ruby',300,'__display__');
+      'curl -sSL https://get.rvm.io | bash -s stable',300,'__display__');
    ($stdout,$stderr)=$handle->cmd_raw('source ~/.rvm/scripts/rvm');
+   ($stdout,$stderr)=$handle->cmd(
+      'rvm install ruby-2.0.0-p648','__display__');
+print "RVM=$stdout<==\n";
+   ($stdout,$stderr)=$handle->cmd(
+      'rvm use 2.0.0','__display__');
+print "RVM NEXT=$stdout<==\n";
+   ($stdout,$stderr)=$handle->cmd_raw('source ~/.rvm/scripts/rvm');
+   ($stdout,$stderr)=$handle->cmd_raw('export PATH='.
+      '/home/ubuntu/.rvm/rubies/ruby-2.0.0-p648/bin:$PATH');
    ($stdout,$stderr)=$handle->cmd(
       'gem install io-console','__display__');
    ($stdout,$stderr)=$handle->cmd(
       'gem install bundler','__display__');
+   ($stdout,$stderr)=$handle->cmd('gem install yard','__display__');
+   ($stdout,$stderr)=$handle->cmd('yard config --gem-install-yri',
+      '__display__');
+   ($stdout,$stderr)=$handle->cmd(
+      'wget http://www.erlang.org/download/otp_src_17.0.tar.gz');
+   ($stdout,$stderr)=$handle->cmd(
+      'tar xf otp_src_17.0.tar.gz');
+   ($stdout,$stderr)=$handle->cwd('otp_src_17.0');
+   ($stdout,$stderr)=$handle->cmd(
+      './configure','__display__');
+   ($stdout,$stderr)=$handle->cmd('sudo '.
+      'make install','__display__');
+   ($stdout,$stderr)=$handle->cwd('~');
    my $download_chef=<<'END';
 
 
@@ -329,21 +355,29 @@ my $configure_chef=sub {
 END
    print $download_chef;sleep 10;
    ($stdout,$stderr)=$handle->cmd(
-      'git clone https://github.com/chef/ohai.git','__display__');
-   ($stdout,$stderr)=$handle->cwd('ohai');
-   ($stdout,$stderr)=$handle->cmd('bundle install','__display__');
-   ($stdout,$stderr)=$handle->cmd('yard config --gem-install-yri','__display__');
-   ($stdout,$stderr)=$handle->cmd('bundle exec rake spec','__display__');
-   ($stdout,$stderr)=$handle->cmd('bundle exec rake install','__display__');
+      'git clone https://github.com/antirez/redis.git','__display__');
+   ($stdout,$stderr)=$handle->cwd('redis');
+   ($stdout,$stderr)=$handle->cmd('sudo make install','__display__');
+   ($stdout,$stderr)=$handle->cmd(
+      '/usr/local/bin/redis-server --daemonize yes','__display__');
    ($stdout,$stderr)=$handle->cwd('~');
    ($stdout,$stderr)=$handle->cmd(
-      'git clone https://github.com/chef/chef.git','__display__');
-   ($stdout,$stderr)=$handle->cwd('chef');
-   ($stdout,$stderr)=$handle->cmd('gem install yard','__display__');
-   ($stdout,$stderr)=$handle->cmd('yard config --gem-install-yri','__display__');
-   ($stdout,$stderr)=$handle->cmd('bundle install','__display__');
-   ($stdout,$stderr)=$handle->cmd('bundle exec rake dependencies:update','__display__');
-   ($stdout,$stderr)=$handle->cmd('bundle exec rake install','__display__');
+      'git clone https://github.com/chef/chef-server.git','__display__');
+   ($stdout,$stderr)=$handle->cwd('chef-server');
+   ($stdout,$stderr)=$handle->cmd('make','__display__');
+   ($stdout,$stderr)=$handle->cmd('sudo chmod -v 777 /var','__display__');
+   ($stdout,$stderr)=$handle->cmd('sudo chmod -v 777 /var/cache','__display__');
+   ($stdout,$stderr)=$handle->cmd('sudo chmod -v 777 /opt','__display__');
+   ($stdout,$stderr)=$handle->cmd('rvm install 2.2.6','__display__');
+   ($stdout,$stderr)=$handle->cmd_raw('rvm --default use 2.2.6');
+   ($stdout,$stderr)=$handle->cmd('env','__display__');
+   ($stdout,$stderr)=$handle->cmd('gem install bundle','__display__');
+print "STDOUT=$stdout<== & STDERR=$stderr<==\n";
+   ($stdout,$stderr)=$handle->cmd('gem install test-kitchen','__display__');
+   ($stdout,$stderr)=$handle->cwd('omnibus');
+   ($stdout,$stderr)=$handle->cmd('bundle install --binstubs','__display__');
+   ($stdout,$stderr)=$handle->cmd('bin/omnibus build chef-server',10000,
+      '__display__');
 
 };
 
@@ -355,6 +389,26 @@ my $standup_chef=sub {
    $type=~s/^(.*?)\s+-[>].*$/$1/;
    my $chef="]T[{select_chef_setup}";
    my $i=$main::aws->{fullauto}->{ImageId}||'';
+   my $os='Ubuntu';
+   if ($os eq 'Ubuntu') {
+      my $region = "wget -qO- http://169.254.169.254/latest/".
+                   "dynamic/instance-identity/document|grep region";
+      $region=`$region`;
+      $region=~s/^.*: ["](.*)["],?\s*$/$1/s;
+      my ($hash,$output,$error)=('','','');
+      ($hash,$output,$error)=run_aws_cmd(
+         "aws ec2 describe-images --owners 099720109477 ".
+         "--filters \"Name=root-device-type,Values=ebs\"".
+         " \"Name=virtualization-type,Values=hvm\" ".
+         "\"Name=architecture,Values=x86_64\" --region=$region");
+      my %images=();
+      foreach my $image (@{$hash->{Images}}) {
+         $images{$image->{CreationDate}}=$image
+            if $image->{Name}=~/ubuntu-trusty/;
+      }
+      my $image_hash=$images{(reverse sort keys %{images})[0]};
+      $i=$image_hash->{ImageId};
+   }
    my $s=$main::aws->{fullauto}->
          {NetworkInterfaces}->[0]->{SubnetId}||'';
    my $g=$main::aws->{fullauto}->
@@ -396,18 +450,41 @@ my $standup_chef=sub {
    my $pemfile=$pem_file;
    $pemfile=~s/\.pem\s*$//s;
    $pemfile=~s/[ ][(]\d+[)]//;
-   if (exists $main::aws->{'Chef.io'}) {
+   if (exists $main::aws->{'Chef.io Server'}) {
       my $g=get_aws_security_id('ChefSecurityGroup');
       my $c="aws ec2 run-instances --image-id $i --count 1 ".
             "--instance-type $type --key-name \'$pemfile\' ".
             "--security-group-ids $g --subnet-id $s";
-      if ($#{$main::aws->{'Chef.io'}}==0) {
-         launch_server('Chef.io',$cnt,$chef,'',$c,
+      if ($#{$main::aws->{'Chef.io Server'}}==0) {
+         launch_server('Chef.io Server',$cnt,$chef,'ubuntu',$c,
                        $configure_chef);
       } else {
-         my $num=$#{$main::aws->{'Chef.io'}}-1;
+         my $num=$#{$main::aws->{'Chef.io Server'}}-1;
+         $c.=' --block-device-mappings "[{\"DeviceName\":\"/dev/sda1\",\"Ebs\":{\"VolumeSize\":20,\"DeleteOnTermination\":true}}]"';
          foreach my $num (0..$num) {
-            launch_server('Chef.io',$cnt++,$chef,'',
+            launch_server('Chef.io Server',$cnt++,$chef,'ubuntu',
+            $c,$configure_chef);
+         }
+      }
+      $cnt=0;
+      if ($#{$main::aws->{'Chef.io Workstation'}}==0) {
+         launch_server('Chef.io Workstation',$cnt,$chef,'ubuntu',$c,
+                       $configure_chef);
+      } else {
+         my $num=$#{$main::aws->{'Chef.io Workstation'}}-1;
+         foreach my $num (0..$num) {
+            launch_server('Chef.io Workstation',$cnt++,$chef,'ubuntu',
+            $c,$configure_chef);
+         }
+      }
+      $cnt=0;
+      if ($#{$main::aws->{'Chef.io Apache Node'}}==0) {
+         launch_server('Chef.io Apache Node',$cnt,$chef,'ubuntu',$c,
+                       $configure_chef);
+      } else {
+         my $num=$#{$main::aws->{'Chef.io Apache Node'}}-1;
+         foreach my $num (0..$num) {
+            launch_server('Chef.io Apache Node',$cnt++,$chef,'ubuntu',
             $c,$configure_chef);
          }
       }
@@ -468,17 +545,39 @@ my $chef_setup_summary=sub {
    #print "HTTPD=$httpd\n";
    my $num_of_servers=0;
    my $ch=$chef;
-   $ch=~s/^.*(\d+)\sServer.*$/$1/;
+   $ch=~s/^.*(\d+)\sChef Server.*$/$1/;
    if ($ch==1) {
-      $main::aws->{'Chef.io'}->[0]=[];
+      $main::aws->{'Chef.io Server'}->[0]=[];
    } elsif ($ch=~/^\d+$/ && $ch) {
       foreach my $n (0..$ch) {
-         $main::aws->{'Chef.io'}=[] unless exists
-            $main::aws->{'Chef.io'};
-         $main::aws->{'Chef.io'}->[$n]=[];
+         $main::aws->{'Chef.io Server'}=[] unless exists
+            $main::aws->{'Chef.io Server'};
+         $main::aws->{'Chef.io Server'}->[$n]=[];
       }
    }
-   $num_of_servers=$ch;
+   my $ws=$chef;
+   $ws=~s/^.*(\d+)\sChef Workstation.*$/$1/;
+   if ($ws==1) {
+      $main::aws->{'Chef.io Workstation'}->[0]=[];
+   } elsif ($ws=~/^\d+$/ && $ws) {
+      foreach my $n (0..$ws) {
+         $main::aws->{'Chef.io Workstation'}=[] unless exists
+            $main::aws->{'Chef.io Workstation'};
+         $main::aws->{'Chef.io Workstation'}->[$n]=[];
+      }
+   }
+   my $an=$chef;
+   $an=~s/^.*(\d+)\sApache Node.*$/$1/;
+   if ($an==1) {
+      $main::aws->{'Chef.io Apache Node'}->[0]=[];
+   } elsif ($an=~/^\d+$/ && $an) {
+      foreach my $n (0..$an) {
+         $main::aws->{'Chef.io Apache Node'}=[] unless exists
+            $main::aws->{'Chef.io Apache Node'};
+         $main::aws->{'Chef.io Apache Node'}->[$n]=[];
+      }
+   }
+   $num_of_servers=$ch+$ws+$an;
    my $cost=int($num_of_servers)*$money;
    my $cents='';
    if ($cost=~/^0\./) {
@@ -622,7 +721,7 @@ END
 
 our $select_chef_setup=sub {
 
-   my @options=('Chef on 4 Servers');
+   my @options=('2 Chef Servers 1 Chef Workstation 1 Apache Node');
    my $chef_setup_banner=<<'END';
 
      ___ _  _ ___ ___     ___      _
